[−][src]Struct tinyvec_string::arraystring::CapacityOverflowError
An error was caused converting another string type into an ArrayString.
This type contains the amount by which the capacity was overflown, and the input string (when possible).
Examples
use std::convert::TryFrom; let result = ArrayString::<[u8; 3]>::try_from("foobar"); assert!(result.is_err()); let err: CapacityOverflowError<&str> = result.unwrap_err(); assert_eq!(err.overflow_amount(), 3); assert_eq!(err.into_inner(), "foobar");
Implementations
impl<S> CapacityOverflowError<S>[src]
pub fn overflow_amount(&self) -> usize[src]
Get the amount by which the capacity was overflown, i.e. the amount of extra capacity that would've been needed to store the string.
pub fn into_inner(self) -> S[src]
Retrieve the input string.
Examples
This could be used to retrieve a String that failed to convert:
use std::convert::TryFrom; let heap_string = String::from("a very long string"); let result = ArrayString::<[u8; 5]>::try_from(heap_string); assert!(result.is_err()); let err: CapacityOverflowError<String> = result.unwrap_err(); let return_of_the_heap_string = err.into_inner(); assert_eq!(return_of_the_heap_string, String::from("a very long string"));